<?php
namespace Teknisa\DF\Controllers;

use Fiscal;
use Fiscal\Logic\ErrosAvisosNF\ErrosAvisosNF;
use Teknisa\DF\Services\Ger03900ErrosAvisosNFService;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager;

class Ger03900ErrosAvisosNFController extends \Teknisa\DF\Controllers\TeknisaCrud {

	protected $dataSourceName = "erroavisonf";
	protected $dataSourceNameToSave = "erroavisonf";
    protected $dataSourceManager;
	protected $service;

    public function __construct($crudService, ParameterBag $parameterBag, Ger03900ErrosAvisosNFService $service, Manager $dataSourceManager) {
    	$this->service = $service;
        parent::__construct($crudService, $parameterBag);
        $this->dataSourceManager = $dataSourceManager;
    }
    
	public function find(DTO\Request\Filter $request, DTO\Response $response) {
		try {
			$this->service->validateDatabase();
		}
		catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
		parent::find($request, $response);
	}
	
    public function save (DTO\Request $request, DTO\Response $response) {
        parent::save($request, $response);
    }
    
    public function saveErrosAvisosUnidade ($request, $response) {
        $this->dataSourceNameToSave = "erroavisonffil";
        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();
        if (!empty($rows)) {
            $rowsToSave = array();
            $rowToSave = array();
            $destino =[];
            
            for($i = 0; $i<count($rows);$i++){
                if($i == 0 || $rows[$i]["CDFILIAL"] !== $rows[$i-1]["CDFILIAL"]){
                    $destino[] = $rows[$i]["CDFILIAL"];
                }
            }
            $condition = array(
                "value" => $destino,
                "operator" => "IN",
                "columnName" => "CDFILIAL"
            );
            $data = $this->findAutoRegister($condition);
            $data = $data->getRows();
            
            $length = count($rows);
            if ($rows[0]['SOBRESCREVER'] == 'N') {
                for ($i=0; $i < $length; $i++) {
                    $flag[$i] = false;
                    $isNew[$i] = true;
                    for ($e=0; $e < count($data); $e++) {
                        if ($data[$e]['CDERROAVISOFIL'] == $rows[$i]['CDERROAVISOFIL']  && $data[$e]['CDFILIAL'] == $rows[$i]['CDFILIAL']) {
                            if ($data[$e]['IDERROAVISOFIL'] == $rows[$i]['IDERROAVISOFIL']) {
                                $flag[$i] = true;
                            } else {
                                $isNew[$i] = false;
                            }
                        }
                    }
                    if($flag[$i]){
                        unset ($rows[$i]);
                    }
                }
            } else {
                $this->deleteForOverwiteRegister($data, $response, $request);
            }
            $index = 0;
            if (!empty($rows)) {
                for($a = 0; $a < $length; $a++){
                    if (!empty($rows[$a])){
                        if ($rows[$a]['SOBRESCREVER'] == 'N') {
                            if (!$flag[$a]) {
                                $rowToSave[$index] = $this->prepareRows($rows[$a], $index, $isNew[$a]);
                                
                                $index++;
                            }
                        } else {
                            $rowToSave[$index] = $this->prepareRows($rows[$a], $index, true);
                            $index++;
                        }
                    }
                }
                try {
                    $dataSet = new DataSet('erroavisonffil', $rowToSave);
                    $requestSave = new DTO\Request\DataSet($dataSet, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
                    $saveRows = $this->crudService->getRowsToSave($requestSave,$this->dataSourceNameToSave);
                    $this->crudService->save('erroavisonffil', $saveRows);
                } catch (\Exception $e) {
                    $response->setError(new Error("". $e->getMessage(), 500));
                }
            } else {
                $response->setError(new DTO\Response\Error("Operação Bloqueada! A parametrização da filial de origem já existe na parametrização das filiais de destino.", 1));
            }
        } else {
            $response->setError(new DTO\Response\Error("Operação Bloqueada! Não há Parametrização para a Filial de Origem.", 1));
        }
    }
    
    public function findAutoRegister($condition) {
        $this->dataSourceName = "erroavisonffil";
        $filterCriteria = new FilterCriteria($this->dataSourceName,[$condition]);
        $data = $this->dataSourceManager->findBy($filterCriteria);
        return $data;
    }
    
    public function deleteForOverwiteRegister($data, $response, $request) {
        $dataSet = new DataSet('erroavisonffil', $data);
        $requestDelete = new DTO\Request\DataSet($dataSet, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        $deleteRows = $requestDelete->getDataSet();
        
        $this->crudService->delete($response, $deleteRows);
    }
    
    public function prepareRows($rows, $index, $isNew) {
        $rowsToSave[$index] = new Row();
        $rowsToSave[$index]->set('__is_new',  $isNew);
        $rowsToSave[$index]->set('CDFILIAL', $rows['CDFILIAL']);
        $rowsToSave[$index]->set('CDERROAVISOFIL', $rows['CDERROAVISOFIL']);
        $rowsToSave[$index]->set('NRORG', $rows['NRORG']);
        if($rows['SOBRESCREVER'] == 'N'){
             return $rowsToSave[$index];
        }
        else{
            $rowsToSave[$index]->set('IDERROAVISOFIL', $rows['IDERROAVISOFIL']);
        }
        return $rowsToSave[$index];
    }
    
    public function deleteRegisters($request, $response){
        $cdfiliais = (array)$request->getRow();
        
        $filter = array(
            "value" => $cdfiliais,
            "operator" => "IN",
            "columnName" => "CDFILIAL"
        );
        $data = $this->findAutoRegister($filter)->getRows();
        
        if(sizeof($data) > 0)
            $this->deleteForOverwiteRegister($data, $response, $request);
    }
    
}