<?php

namespace Teknisa\DF\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\DF\Services\RegrasTributariasService as RegrasTributariasService;

class RegrasTributarias extends \Teknisa\DF\Controllers\TeknisaCrud
{
    protected $dataSourceName = 'sys_cenario';
    protected $environment;
    protected $entityManager;
    protected $regrasTributariasService;
    protected $parameterBagColumns = array(
        'P_CDFILIAL'
    );
    
    public function __construct($crudService, ParameterBag $parameterBag, $environment, $entityManager, RegrasTributariasService $regrasTributariasService) {
        parent::__construct($crudService, $parameterBag);
        $this->environment      = $environment;
        $this->entityManager    = $entityManager;
        $this->regrasTributariasService = $regrasTributariasService;
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response) {

        $this->dataSourceName = 'sys_cenario';
        
        parent::find($request, $response);
        
    }
    
    public function find_estado(DTO\Request\Filter $request, DTO\Response $response) {

        $this->dataSourceName = 'estado_filtro';

        parent::find($request, $response);
        
    }
    
    public function find_material(DTO\Request\Filter $request, DTO\Response $response) {

        $this->dataSourceName = 'sys_material';
        
        parent::find($request, $response);
        
    }
    
    public function save_material(Row $row, Response $response) {
        try{
            $row  = $row->getRow();
           
            $nrorg           = $row["NRORG"];
            $cod_prod        = $row["COD_PROD"];
            $idcenario       = $row['ID_CENARIO'];
            $idativo         = $row["IDATIVO"];
                
            $dados = $this->regrasTributariasService->save_material($nrorg, $idcenario, $cod_prod, $idativo);
            
            $response->addMessage(new DTO\Response\Message("Item salvo com sucesso."));
            
        }catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function find_icms(DTO\Request\Filter $request, DTO\Response $response) {

        $this->dataSourceName = 'sys_icms';

        parent::find($request, $response);
        
    }
    
    public function find_ipi(DTO\Request\Filter $request, DTO\Response $response) {

        $this->dataSourceName = 'sys_ipi';

        parent::find($request, $response);
        
    }
    
    public function find_pis(DTO\Request\Filter $request, DTO\Response $response) {

        $this->dataSourceName = 'sys_pis';

        parent::find($request, $response);
        
    }
    
    public function find_cofins(DTO\Request\Filter $request, DTO\Response $response) {

        $this->dataSourceName = 'sys_cofins';

        parent::find($request, $response);
        
    }
    
    public function find_icmsst(DTO\Request\Filter $request, DTO\Response $response) {

        $this->dataSourceName = 'sys_icmsst';

        parent::find($request, $response);
        
    }
    
    /**
     * @param Row $row
     * @param Response $response
     */
    public function getCodProd(Row $row, Response $response)
    {
        try {
            // Timeout Ilimitado
            //set_time_limit(0);
            
            $row  = $row->getRow();
            //var_dump($row['ID_CENARIO']);die;

            $nrorg           = $row['NRORG'];
            $idcenario       = $row['ID_CENARIO'];
            
            $dados = $this->regrasTributariasService->getCodProd($nrorg, $idcenario);
            //var_dump($dados);die;

            $response->addDataSet(new DataSet('getCodProd', ['dados' => $dados]));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
}