<?php

namespace Teknisa\DF\Modules\Entrada\Services;

/* Bibliotecas */
use NFePHP\CTe\Tools as CTe;
use NFePHP\Common\Certificate;
use NFePHP\Common\UFList;
use NFePHP\Common\Soap\SoapCurl;
use NFePHP\NFe\Common\Config;
use Doctrine\ORM\EntityManager;
use Geral\Entities\Empresa;
use Geral\Entities\Inscrestad;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\Masks\Masks;
use Teknisa\Libs\Util\Utilities;

/* Classes DF */
use Teknisa\DF\Modules\Entrada\Factories\Dfe71000SincronizacaoCTEFactory;
use Teknisa\DF\Services\NFe\ReturnNFe;

class Dfe71000SincronizacaoCTEService {
    
    protected $sincronizacaoCTEFactory;
    protected $fileUploader;
    protected $entityManager;
    protected $environment;
    protected $transmissaoService;
    protected $cteTools;
    protected $aResposta = [];
    protected $notasDownload;
    protected $quantidadeEventos;
    protected $chaves = [];
    protected $importacaoPendente = [];
    protected $sincronizadas;
    protected $notasInexistentes = [];
    protected $erros = [];
    
    public function __construct(Dfe71000SincronizacaoCTEFactory $sincronizacaoCTEFactory, $entityManager, $environment, $fileServerLink) {
        $this->sincronizacaoCTEFactory = $sincronizacaoCTEFactory;
        $this->entityManager        = $entityManager;
        $this->environment          = $environment;
        $this->masks = new Masks();
        $this->fileServerLink       = $fileServerLink;
        set_time_limit(0);
    }
    
    public function sincronizarCte($config){
        $config = json_decode(json_encode($config));
        $object_config = $config;
        
    	$connection = $this->entityManager->getConnection();
    	$tools = $this->getToolsNFe($object_config->ie, $object_config->cnpj, json_encode($config));
    	$this->cteTools = $tools;
        
        $cnpj = $object_config->cnpj;
        $existeNotas = true;
        
        $ultNSU = $this->sincronizacaoCTEFactory->maxNrNsu($cnpj, $object_config->tpAmb);
        
        $ultNSU = (!$ultNSU) ? 0 : $ultNSU;
        $ultNSU = str_pad($ultNSU, 15, "0", STR_PAD_LEFT);

        $count  = 0; $this->quantidadeEventos = 0; $this->notasDownload = 0; $this->sincronizadas = 0;
        $this->chaves = []; $this->notasInexistentes = []; $this->importacaoPendente = []; $notas = []; $this->erros = [];
        
        do {
            try {
                $this->buscaNotasSefaz($ultNSU, 0);
                $notas = $this->aResposta;
                
                $connection->beginTransaction();
                $this->trataRetornoLoteSincronizacao($cnpj, $notas, $object_config);
                $connection->commit();
            }
            catch(\Exception $e) {
                if($connection->isTransactionActive()) $connection->rollback();
                
                if($e->getCode() === 137) {
                    $existeNotas = false;
                }
                else if(strpos($e->getMessage(), "Certificate") !== false) {
                    throw new \Exception("Certificado vencido. ".$e->getMessage());
                }
                
                $this->erros[] = $e->getMessage() . ' - '. $e->getCode();
            }
            
            if(isset($notas["ultNSU"]) && isset($notas["maxNSU"]) && isset($notas["cStat"])) {
               
               if($notas["cStat"] == 656 || $count > 11) { //Finaliza a sincronização com Consumo indevido ou 12 consultas a sefaz evitando erro de consumo indevido.
                  $existeNotas = false;
                  break;
               }
                else if($notas["ultNSU"] != $notas["maxNSU"]) {
                    $ultNSU = $notas["ultNSU"];
                    $count++;
                }
                else {
                    $existeNotas = false;
                    break;
                }
            }
            else {
                $existeNotas = false;
                break;
            }
        } while($existeNotas === true);
        
        $this->downloadAutomaticoXML($cnpj, $object_config->tpAmb);
        
        $chavesRetorno = [];
        foreach($this->chaves as $chave) {
            $chavesRetorno[] = $chave;
        }
        
        return array("CHAVES" => $chavesRetorno, "DADOS" => $this->calculaDadosSincronizacao(), "ERROS" => $this->erros);
    }
    
    public function montaConfigCte($empresa, $organizacao, $inscricao, $cnpj) {
        $objEmpresa = $this->entityManager->getRepository('Geral\Entities\Empresa')->findOneBy(['cdempresa' => $empresa, 'nrorg' => $organizacao]);
        
        $cnpj = $this->masks->removeMascaraInscricao($cnpj);
        
        $razaosocial = trim($this->masks->replaceCharsForParser($this->masks->removeAcentos($objEmpresa->getNmrazsocempr())));
        $nomefantasia = $razaosocial;
        
        if(!empty($inscricao)) {
            if(strpos("ISENTO",strtoupper($inscricao)) || strpos("ISENTA", strtoupper($inscricao))){
                $ie = "ISENTO";
            }
            else{
                $ie = str_replace(" ", "", $this->masks->removeMascaraInscricao($this->masks->removeAcentos($inscricao)));
            }
            $inscrestad = self::getInscricaoEstadual($empresa, $organizacao, $inscricao, $cnpj);
        }
        else{
            $inscrestad = $this->entityManager->getRepository('Geral\Entities\Inscrestad')->findBy(['cdempresa' => $empresa, 'nrinsjuriest' => $cnpj, 'nrorg' => $organizacao]);

            if(empty($inscrestad)) throw new \Exception('Inscrição Estadual não Encontrada');
            else{
                $ieComCertificados = array_filter($inscrestad, function($certRow){
                    return !empty($this->sincronizacaoCTEFactory->getCertificado($certRow->getCdinscesta(), $certRow->getNrinsjuriest()));
    		    });
    		    
    		    if(empty($ieComCertificados)) throw new \Exception('Certificado Digital não Encontrado');
    		    else{
    		        $inscrestad = $ieComCertificados[0];
    		        $ie = $inscrestad->getCdinscesta();
    		    }
            }
        }
        $tpAmb = (int) $inscrestad->getIdtpambientenfe(); // 1-Produção 2-Homologação
        $siglaUF = $inscrestad->getSgestado();
        $config = [
            "tpAmb"       => $tpAmb,
            "razaosocial" => $nomefantasia,
            "siglaUF"     => $siglaUF,
            "cnpj"        => $cnpj,
            "cpf"         => null,
            "schemes"     => "PL_CTe_300a",
            "versao"      => "3.00",
            "ie"          => $ie,
            "cdempresa"   => $empresa,
            "nrorg"       => $this->environment->getNrOrgTrab()];
        
        $resp = Config::validate(json_encode($config));
        return $resp;
    }
     
     /**
     * Retorna inscricao estadual da empresa
     * @param $empresa
     * @param $cdinscestad
     * @param $organizacao
     * @return Inscrestad
     * @throws \Exception
     */
    private function getInscricaoEstadual($empresa, $organizacao, $inscricao, $cnpj){
        $inscrestad = $this->entityManager->getRepository('Geral\Entities\Inscrestad')->findOneBy(
                        ['cdempresa' => $empresa, 'nrinsjuriest' => $cnpj, 'nrorg' => $organizacao, 'cdinscesta'=> $inscricao]);
        
        if (!($inscrestad instanceof Inscrestad)) throw new \Exception('Inscrição Estadual não Encontrada');

        return $inscrestad;
    }
    
    private function downloadAutomaticoXML($cnpj, $tpAmb) {
        $cteDownload = $this->sincronizacaoCTEFactory->getNotasDownloadAutomatico($cnpj, $tpAmb);
        
        if($cteDownload) {
    		foreach($cteDownload as $notaCte) {
    		    $xml                    = new \DOMDocument();
                $xml->preservWhiteSpace = false; //elimina espacos em branco
                $xml->formatOutput      = false;
                $xml->loadXML($notaCte['DSARQXMLNSUCTE']);
                
                $protCTe = $xml->getElementsByTagName('protCTe')->item(0);
                
        		if($protCTe) {
        		    $chaveCTe = $protCTe->getElementsByTagName('infProt')->item(0)->getElementsByTagName('chCTe')->item(0)->nodeValue;
        		    
        		    if($chaveCTe) {
        			    try {
            			    if($this->environment->getNrOrgTrab() != 1) {
        		                if(!isset($this->sincronizacaoCTEFactory->existeNotaRelac($chaveCTe)[0])) {
                    			    $this->sincronizacaoCTEFactory->zGravaFile($tpAmb, $chaveCTe.'.xml', $notaCte['DSARQXMLNSUCTE'], 'importar/', $chaveCTe);
                    			    $this->sincronizacaoCTEFactory->atualizaCTeBaixado($notaCte['NRINSJURPESQ'], $notaCte['IDTPAMBIENTE'], $notaCte['NRNSU'], $this->environment->getNrOrgTrab());
                			        $this->notasDownload++;
        		                }
                			}
                			else {
                			    $diretorioIntegracao = $this->sincronizacaoCTEFactory->getDiretorioIntegracao();
                			    
            			        if(isset($diretorioIntegracao[0])) {
            			            if($diretorioIntegracao[0]["DSDIRIMPARQXML"] != null) {
            			                if(!is_file($diretorioIntegracao[0]["DSDIRIMPARQXML"]."/$chaveCTe.xml")) {
                			                file_put_contents($diretorioIntegracao[0]["DSDIRIMPARQXML"]."/$chaveCTe.xml", $notaCte['DSARQXMLNSUCTE']);
                			                $this->notasDownload++;
            			                }
            			            }
            			        }
                			}
        			    } catch(\Exception $e) {
        		            $this->erros[] = $e->getMessage() . ' - '. $e->getCode();
        		        }
        		    }
    			}
    		}
        }
    }

    public function preencheTabelasSincManifestacao($sincronizacao, $config, $ultNSU=''){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $countNotas = count($sincronizacao["aDoc"]);
            $nsuLogica = (intval($ultNSU) - $countNotas);
            
            foreach ($sincronizacao['aDoc'] as $doc) {
                
                $nsuLogica++;
                $nsu = (isset($doc['NSU'])) ? $doc['NSU'] : str_pad($nsuLogica , 15 , '0' , STR_PAD_LEFT);
                $ultNSU = $sincronizacao['ultNSU'];
                $xml                    = new \DOMDocument();
                $xml->preservWhiteSpace = false; //elimina espacos em branco
                $xml->formatOutput      = false;
                $xml->loadXML($doc['doc']);
                $arrayXml = $this->converteXmlParaArray($xml->saveXML());
                
                if(strtolower(substr($doc['schema'], 0, 7)) == strtolower('procCTe') ||
                   strtolower(substr($doc['schema'], 0, 9)) == strtolower('cteOSProc')) {
                    
                    $chave = $arrayXml['protCTe']['infProt']['chCTe'];
                    
                    if(!empty($chave)) {
                	    $this->preencheNotaCte($config, $arrayXml, $chave, $nsu, $doc['schema']);
                	    $this->pesquisaCteAceitos($chave);
                    }
                }
                else if(strtolower(substr($doc['schema'], 0, 13)) == strtolower('procEventoCTe')) {
                    
                    $chave = $arrayXml['retEventoCTe']['infEvento']['chCTe'];
                    
                    if(!empty($chave)) {
                        $this->preencheNotaCte($config, $arrayXml, $chave, $nsu, 'procEventoCTe');
                        $this->verificaEventoCte($xml);
                    }
                }
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    private function preencheNotaCte($config, $arrayXml, $chave, $nsu, $schema){
        $existeNotaCte = $this->sincronizacaoCTEFactory->existeNotaCte($nsu, $config->cnpj, $config->tpAmb);
        $dadosCte = [];
        
             if(strtolower(substr($schema, 0, 7)) == strtolower('procCTe'))   $this->preparaDadosProcCTe($dadosCte, $chave, $arrayXml, $nsu, $config->cnpj);
        else if(strtolower(substr($schema, 0, 9)) == strtolower('cteOSProc')) $this->preparaDadosCTeOSProc($dadosCte, $chave, $arrayXml, $nsu, $config->cnpj);
        else if($schema == 'procEventoCTe')                                   $this->preparaDadosProcEventoCTe($dadosCte, $chave, $arrayXml, $nsu, $config->cnpj);
        
        if(empty($existeNotaCte)){
            $this->atualizaNSUFaltante($config->cnpj, $config->tpAmb, $nsu);
            
            if(!empty($dadosCte)) {
                $this->sincronizacaoCTEFactory->incluirNotaCte($dadosCte);
            }
        }
        else if(!empty($dadosCte)){
            $this->sincronizacaoCTEFactory->atualizaNotaCte($dadosCte);
        }
    }
    
    private function preparaDadosProcCTe(&$dadosCte, $nracessocte, $arrayXml, $nsu, $cnpj) {
        $CTe = $arrayXml['CTe'];
        $infCte = $CTe['infCte'];
        $ide = $infCte['ide'];
        $emit = $infCte['emit'];
        $rem   = isset($infCte['rem']) ? $infCte['rem'] : null;
        $exped = isset($infCte['exped']) ? $infCte['exped'] : null;
        $receb = isset($infCte['receb']) ? $infCte['receb'] : null;
        $dest = isset($infCte['dest']) ? $infCte['dest'] : null;
        
        $idtpambiente = $ide['tpAmb'];
        $dtEmissao = date('d/m/Y', strtotime(substr($ide["dhEmi"], 0, 19)));
        $vrtotalprest = $infCte['vPrest']['vTPrest'];
        $vrreceber = $infCte['vPrest']['vRec'];
        $cdstatus = $arrayXml['protCTe']['infProt']['cStat'];
        $dsstatus = $arrayXml['protCTe']['infProt']['xMotivo'];
        $nrinsjurreme = null; $nmrazsocreme = null; $nrinsjurexpe = null; $nmrazsocexpe = null; $nrinsjurrece = null;
        $nmrazsocrece = null; $nrinsjurdest = null; $nmrazsocdest = null;
        
        /*EMITENTE*/
            $nrinsjuremit = $emit['CNPJ'];
            $nmrazsocemit = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($emit['xNome'])));
        /*REMETENTE*/
        if(isset($rem)) {
            $nrinsjurreme = isset($rem['CNPJ']) ? $rem['CNPJ'] : $rem['CPF'];
            $nmrazsocreme = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($rem['xNome'])));
        }
        /*EXPEDIDOR*/
        if(isset($exped)) {
            $nrinsjurexpe = isset($exped['CNPJ']) ? $exped['CNPJ'] : $exped['CPF'];
            $nmrazsocexpe = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($exped['xNome'])));
        }
        /*RECEBEDOR*/
        if(isset($receb)) {
            $nrinsjurrece = isset($receb['CNPJ']) ? $receb['CNPJ'] : $receb['CPF'];
            $nmrazsocrece = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($receb['xNome'])));
        }
        /*DESTINATÁRIO*/
        if(isset($dest)) {
            $nrinsjurdest = isset($dest['CNPJ']) ? $dest['CNPJ'] : $dest['CPF'];
            $nmrazsocdest = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($dest['xNome'])));
        }
        /*TOMADOR*/
        if(isset($ide['toma3'])) {
            $idtomador = $ide['toma3']['toma'];
            
            switch($ide['toma3']['toma']) {
                case 0:
                    $nrinsjurtoma = $nrinsjurreme;
                    $nmrazsoctoma = $nmrazsocreme;
                    break;
                case 1:
                    $nrinsjurtoma = $nrinsjurexpe;
                    $nmrazsoctoma = $nmrazsocexpe;
                    break;
                case 2:
                    $nrinsjurtoma = $nrinsjurrece;
                    $nmrazsoctoma = $nmrazsocrece;
                    break;
                case 3:
                    $nrinsjurtoma = $nrinsjurdest;
                    $nmrazsoctoma = $nmrazsocdest;
                    break;
            }
        } else {
            $idtomador = 4;
            $nrinsjurtoma = isset($ide['toma4']['CNPJ']) ? $ide['toma4']['CNPJ'] : $ide['toma4']['CPF'];
            $nmrazsoctoma = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($ide['toma4']['xNome'])));
        }
        
        $dadosCte = ["NRACESSOCTE"  => $nracessocte,  "NSU"          => $nsu,          "NRINSJURPESQ" => $cnpj,         "IDTPAMBIENTE" => $idtpambiente,
                     "DTEMISSAO"    => $dtEmissao,    "VRTOTALPREST" => $vrtotalprest, "VRRECEBER"    => $vrreceber,    "CDSTATUS"     => $cdstatus,
                     "DSSTATUS"     => $dsstatus,     "NRINSJUREMIT" => $nrinsjuremit, "NMRAZSOCEMIT" => $nmrazsocemit, "NRINSJURREME" => $nrinsjurreme,
                     "NMRAZSOCREME" => $nmrazsocreme, "NRINSJUREXPE" => $nrinsjurexpe, "NMRAZSOCEXPE" => $nmrazsocexpe, "NRINSJURRECE" => $nrinsjurrece,
                     "NMRAZSOCRECE" => $nmrazsocrece, "NRINSJURDEST" => $nrinsjurdest, "NMRAZSOCDEST" => $nmrazsocdest, "IDTOMADOR"    => $idtomador,
                     "NRINSJURTOMA" => $nrinsjurtoma, "NMRAZSOCTOMA" => $nmrazsoctoma];
    }
    
    private function preparaDadosCTeOSProc(&$dadosCte, $nracessocte, $arrayXml, $nsu, $cnpj) {
        $CTeOS = $arrayXml['CTeOS'];
        $infCte = $CTeOS['infCte'];
        $ide = $infCte['ide'];
        $emit = $infCte['emit'];
        $toma = isset($infCte['toma']) ? $infCte['toma'] : null;
        
        $idtpambiente = $ide['tpAmb'];
        $dtEmissao = date('d/m/Y', strtotime(substr($ide["dhEmi"], 0, 19)));
        $vrtotalprest = $infCte['vPrest']['vTPrest'];
        $vrreceber = $infCte['vPrest']['vRec'];
        $cdstatus = $arrayXml['protCTe']['infProt']['cStat'];
        $dsstatus = $arrayXml['protCTe']['infProt']['xMotivo'];
        
        /*EMITENTE*/
        $nrinsjuremit = $emit['CNPJ'];
        $nmrazsocemit = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($emit['xNome'])));
        /*REMETENTE*/
        $nrinsjurreme = null; $nmrazsocreme = null;
        /*EXPEDIDOR*/
        $nrinsjurexpe = null; $nmrazsocexpe = null;
        /*RECEBEDOR*/
        $nrinsjurrece = null; $nmrazsocrece = null;
        /*DESTINATÁRIO*/
        $nrinsjurdest = null; $nmrazsocdest = null;
        /*TOMADOR*/
        $idtomador = null; $nrinsjurtoma = null; $nmrazsoctoma = null;
        if(isset($toma)) {
            $nrinsjurtoma = isset($ide['toma']['CNPJ']) ? $ide['toma']['CNPJ'] : $ide['toma']['CPF'];
            $nmrazsoctoma = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($ide['toma']['xNome'])));
        }
        
        $dadosCte = ["NRACESSOCTE"  => $nracessocte,  "NSU"          => $nsu,          "NRINSJURPESQ" => $cnpj,         "IDTPAMBIENTE" => $idtpambiente,
                     "DTEMISSAO"    => $dtEmissao,    "VRTOTALPREST" => $vrtotalprest, "VRRECEBER"    => $vrreceber,    "CDSTATUS"     => $cdstatus,
                     "DSSTATUS"     => $dsstatus,     "NRINSJUREMIT" => $nrinsjuremit, "NMRAZSOCEMIT" => $nmrazsocemit, "NRINSJURREME" => $nrinsjurreme,
                     "NMRAZSOCREME" => $nmrazsocreme, "NRINSJUREXPE" => $nrinsjurexpe, "NMRAZSOCEXPE" => $nmrazsocexpe, "NRINSJURRECE" => $nrinsjurrece,
                     "NMRAZSOCRECE" => $nmrazsocrece, "NRINSJURDEST" => $nrinsjurdest, "NMRAZSOCDEST" => $nmrazsocdest, "IDTOMADOR"    => $idtomador,
                     "NRINSJURTOMA" => $nrinsjurtoma, "NMRAZSOCTOMA" => $nmrazsoctoma];
    }
    
    private function preparaDadosProcEventoCTe(&$dadosCte, $nracessocte, $arrayXml, $nsu, $cnpj) {
        $cteAceitos = $this->sincronizacaoCTEFactory->pesquisaCteAceitos($nracessocte);
        
        $idtpambiente = $arrayXml['retEventoCTe']['infEvento']['tpAmb'];
        $cdstatus = $arrayXml['retEventoCTe']['infEvento']['cStat'];
        $dsstatus = $arrayXml['retEventoCTe']['infEvento']['xMotivo'];
            
        if($cteAceitos) {
            $dtEmissao = $cteAceitos['DTEMISSAO'];
            $vrtotalprest = $cteAceitos['VRTOTALPREST'];
            $vrreceber = $cteAceitos['VRRECEBER'];
            
            /*EMITENTE*/
            $nrinsjuremit = $cteAceitos['NRINSJUREMIT'];
            $nmrazsocemit = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($cteAceitos['NMRAZSOCEMIT'])));
            /*REMETENTE*/
            $nrinsjurreme = $cteAceitos['NRINSJURREME'];
            $nmrazsocreme = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($cteAceitos['NMRAZSOCREME'])));
            /*EXPEDIDOR*/
            $nrinsjurexpe = $cteAceitos['NRINSJUREXPE'];
            $nmrazsocexpe = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($cteAceitos['NMRAZSOCEXPE'])));
            /*RECEBEDOR*/
            $nrinsjurrece = $cteAceitos['NRINSJURRECE'];
            $nmrazsocrece = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($cteAceitos['NMRAZSOCRECE'])));
            /*DESTINATÁRIO*/
            $nrinsjurdest = $cteAceitos['NRINSJURDEST'];
            $nmrazsocdest = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($cteAceitos['NMRAZSOCDEST'])));
            /*TOMADOR*/
            $idtomador = $cteAceitos['IDTOMADOR'];
            $nrinsjurtoma = $cteAceitos['NRINSJURTOMA'];
            $nmrazsoctoma = trim($this->replaceCharsForParserInverted($this->masks->removeAcentos($cteAceitos['NMRAZSOCTOMA'])));
            
            $dadosCte = ["NRACESSOCTE"  => $nracessocte,  "NSU"          => $nsu,          "NRINSJURPESQ" => $cnpj,         "IDTPAMBIENTE" => $idtpambiente,
                         "DTEMISSAO"    => $dtEmissao,    "VRTOTALPREST" => $vrtotalprest, "VRRECEBER"    => $vrreceber,    "CDSTATUS"     => $cdstatus,
                         "DSSTATUS"     => $dsstatus,     "NRINSJUREMIT" => $nrinsjuremit, "NMRAZSOCEMIT" => $nmrazsocemit, "NRINSJURREME" => $nrinsjurreme,
                         "NMRAZSOCREME" => $nmrazsocreme, "NRINSJUREXPE" => $nrinsjurexpe, "NMRAZSOCEXPE" => $nmrazsocexpe, "NRINSJURRECE" => $nrinsjurrece,
                         "NMRAZSOCRECE" => $nmrazsocrece, "NRINSJURDEST" => $nrinsjurdest, "NMRAZSOCDEST" => $nmrazsocdest, "IDTOMADOR"    => $idtomador,
                         "NRINSJURTOMA" => $nrinsjurtoma, "NMRAZSOCTOMA" => $nmrazsoctoma];
        }
        else {
            $dadosCte = ["NRACESSOCTE"  => $nracessocte,  "NSU"          => $nsu,          "NRINSJURPESQ" => $cnpj,         "IDTPAMBIENTE" => $idtpambiente,
                         "DTEMISSAO"    => null,          "VRTOTALPREST" => null,          "VRRECEBER"    => null,          "CDSTATUS"     => $cdstatus,
                         "DSSTATUS"     => $dsstatus,     "NRINSJUREMIT" => null,          "NMRAZSOCEMIT" => null,          "NRINSJURREME" => null,
                         "NMRAZSOCREME" => null,          "NRINSJUREXPE" => null,          "NMRAZSOCEXPE" => null,          "NRINSJURRECE" => null,
                         "NMRAZSOCRECE" => null,          "NRINSJURDEST" => null,          "NMRAZSOCDEST" => null,          "IDTOMADOR"    => null,
                         "NRINSJURTOMA" => null,          "NMRAZSOCTOMA" => null];
        }
    }
    
    private function pesquisaCteAceitos($nracessocte) {
        $cteAceitos = $this->sincronizacaoCTEFactory->pesquisaCteAceitos($nracessocte);
        if($cteAceitos) $this->sincronizacaoCTEFactory->atualizaCteAceitos($cteAceitos);
    }
    
    private function verificaEventoCte($xml) {
        $tpEvento = $xml->getElementsByTagName('tpEvento')->item(0)->nodeValue;
        
        if($tpEvento) {
            if($tpEvento === '110110' || $tpEvento === '110111') $this->quantidadeEventos++;
        }
    }
    
    public function gerarXmlCteManifestacao($ctes, $config){
        $object_config = json_decode(json_encode($config));
        
    	if(!isset($this->cteTools)) {
    		$tools = $this->getToolsNFe($object_config->ie, $object_config->cnpj, json_encode($config));
    		$this->cteTools = $tools;
        }
        
        $existeCTe = false;
        $dir = sys_get_temp_dir();
        $tpAmb = $object_config->tpAmb;
        $zipname = $dir.DIRECTORY_SEPARATOR.uniqid().'XMLs.zip';
        $zip = new \ZipArchive();

        if($zip->open($zipname, \ZipArchive::CREATE) === true){
            foreach ($ctes as $cte) {
                $chave = $cte['NRACESSOCTE'];
                
                $xmlCTe = $this->sincronizacaoCTEFactory->getXMLCTe($object_config->cnpj, $tpAmb, $cte['NRNSU']);
                
                if(!empty($xmlCTe)) {
                    file_put_contents($dir.DIRECTORY_SEPARATOR.$chave.'.xml', $xmlCTe['DSARQXMLNSUCTE']);
                    $xmlZip = $dir.DIRECTORY_SEPARATOR.$chave.'.xml';
                }
                else{
                    $xmlZip = $this->downloadCTe($cte['NRNSU'], $dir, $chave, $object_config);
                }
                
                if(!empty($xmlZip)) {
                    $existeCTe = true;
                    $zip->addFromString('CTe-'.$chave.'.xml' , file_get_contents($xmlZip));
                    unlink($xmlZip);
                }
            }
            $zip->close();
        }
        
        if(!$existeCTe) return false;
        else            return $zipname;
    }
    
    private function buscaNotasSefaz($ultNSU, $numNSU){
        $this->aResposta = [];
        $resp = $this->cteTools->sefazDistDFe($ultNSU, $numNSU, 'AN');
        
        $this->aResposta = ReturnNFe::readReturnSefaz('NfeDistribuicaoDFe', $resp);
        
        return $this->aResposta;
    }

    private function downloadCTe($nsu, $dir, $chave, $object_config){
        try{
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $this->buscaNotasSefaz(0, $nsu);
            $notas = $this->aResposta;
            
            $this->trataRetornoLoteSincronizacao($object_config->cnpj, $notas, $object_config);
            $connection->commit();
        } catch(\Exception $e) {
            $connection->rollback();
            throw new \Exception($e->getMessage(), $e->getCode());
        }
        
        if(isset($this->aResposta)) {
            if(!empty($this->aResposta["aDoc"])) {
                file_put_contents($dir.DIRECTORY_SEPARATOR.$chave.'.xml', $this->aResposta["aDoc"][0]["doc"]);
                return $dir.DIRECTORY_SEPARATOR.$chave.'.xml';
            }
            else return false;
        }
        else return false;
    }
    
    /**
    *   Converte uma string Xml para um array.
    *    
    *   @param string $stringXml
    *   return  array - Array do XML
    */
    private function converteXmlParaArray($stringXml) {
        $xml = simplexml_load_string($stringXml,null,LIBXML_NOCDATA);
        $json = json_encode($xml);
        $array = json_decode($json,TRUE);
        return $array;
    }
    
    private function calculaDadosSincronizacao() {
        $dadosSincronizacao = [];
        $dadosSincronizacao["QTNOTAS"]   = "Foram encontradas ".count($this->chaves)." nota(s) de CTE.";
        $dadosSincronizacao["QTEVENTOS"] = "Foram encontrados $this->quantidadeEventos do tipo cancelamento ou carta de correção.";
        $dadosSincronizacao["QTINEXIST"] = "Existem ".count($this->notasInexistentes)." notas fiscais destinadas inexistentes no sistema.";
        $dadosSincronizacao["QTIMPORTP"] = "Existem ".count($this->importacaoPendente)." notas fiscais com importação de XML pendente.";
        $dadosSincronizacao["SINCRONIZ"] = "Foram atualizadas $this->sincronizadas notas fiscais existentes no sistema";
        $dadosSincronizacao["NOTASDOWN"] = "Download de $this->notasDownload arquivos XML realizado com sucesso";
        return $dadosSincronizacao;
    }

	public function getToolsNFe($ie, $cnpj, $config) {
	    
	    /*Função criada para buscar o cnpj vinculada a inscrição estadual, de forma que ele sempre busque o certificado correto*/
	    
	    $cnpjInscEstad = $this->sincronizacaoCTEFactory->getCNPJCertificado($ie, json_decode($config)->cdempresa);
	    $certificado = $this->sincronizacaoCTEFactory->getCertificado($ie, $cnpjInscEstad);
		$certificadoEncontrado = false;
		
		foreach($certificado as $cert) {
		    if($cert["DSDIRCERTIFICADO"]) {
		        $certificadoEncontrado = true;
		        $certify = $cert;
		    }
		}
		
		if($certificadoEncontrado == false) throw new \Exception("Certificado Digital não encontrado.");
		
        $senha = $this->decrypt($certify["CDSENHACERTIF"]);
		$certificado = $certify["DSDIRCERTIFICADO"];
		$certificado = explode('>', $certificado);
        $certificado = file_get_contents($this->fileServerLink.$certificado[1]);
        
        try {
            $certificado = Certificate::readPfx($certificado, $senha);
        }
        catch(\Exception $e) {
            throw new \Exception("Senha do certificado incorreta!");
        }
        $tools = new CTe($config, $certificado);
        
        $tools2 = new SoapCurl($certificado);
        $tools2->httpVersion('1.1');
        
        $tools->loadSoapClass($tools2);
        
        return $tools;
	}
	
	private function decrypt($text, $salt = 'teknisa') {
	    return Utilities::zeedhiDecrypt($text, $salt);
    }
    
    private function trataRetornoLoteSincronizacao($cnpj, $notas, $object_config) {
        $cStat = $notas["cStat"];
        $xMotivo = $notas["xMotivo"];
        $ultNSU = $notas["ultNSU"];
        
        if($cStat == 138) { //Documento localizado.
            if(isset($notas["aDoc"])) {
                $this->preencheTabelasSincManifestacao($notas, $object_config, $ultNSU);
                $countNotas = count($notas["aDoc"]);
                $nsuLogica = (intval($ultNSU) - $countNotas);
                
                foreach($notas["aDoc"] as $notaCte) {
                    $nsuLogica++;
                    $arrayXml = $this->converteXMLParaArray($notaCte['doc']);
                    $nsu = (isset($notaCte["NSU"])) ? $notaCte["NSU"] : str_pad($nsuLogica , 15 , '0' , STR_PAD_LEFT);
                    $tipoNota = false; $tipoEvento = false;
                    
                    //procCTe e cteOSProc
                    if(strtolower(substr($notaCte["schema"], 0, 7)) == strtolower('procCTe') ||
                       strtolower(substr($notaCte["schema"], 0, 9)) == strtolower('cteOSProc')) {
                        
                        $protCTe = $arrayXml['protCTe'];
                        $infProt = $protCTe['infProt'];
                        $chave = $infProt['chCTe'];
                        $tipoNota = true;
                    	
                    	if($infProt['cStat'] !== 110) {
                    	    $existeNotaCte = $this->sincronizacaoCTEFactory->existeNotaCte($nsu, $cnpj, $infProt['tpAmb']);
                    	    
                    	    if($existeNotaCte) {
                    	        $this->verificaXmlCte($cnpj, $infProt['tpAmb'], $nsu, $notaCte['doc'], null, null);
                    	    }
                    	}
                    	
                    	if(!isset($this->chaves[$chave])) {
            			    $this->chaves[$chave] = $chave; //Apenas ctes do tipo "procCTe" ou "cteOSProc" são considerados para retorno.
            			}
                    }
                    //procEventoCTe
                    else if(strtolower(substr($notaCte["schema"], 0, 13)) == strtolower('procEventoCTe')) {
                    	$retEventoCTe = $arrayXml['retEventoCTe'];
                    	$infEvento = $retEventoCTe['infEvento'];
                    	$chave = $infEvento['chCTe'];
                    	$tipoEvento = true;
                    	
                    	if($infEvento['cStat'] !== 110) {
                    	    $existeNotaCte = $this->sincronizacaoCTEFactory->existeNotaCte($nsu, $cnpj, $infEvento['tpAmb']);
                    	    
                    	    if($existeNotaCte) {
                        	    $this->verificaXmlCte($cnpj, $infEvento['tpAmb'], $nsu, $notaCte['doc'], $infEvento['tpEvento'], $infEvento['xEvento']);
                    	    }
                    	}
                    }
                    
                    if($tipoNota === true || $tipoEvento === true) {
                        $notaSincronizada = $this->sincronizacaoCTEFactory->getNotaSincronizada($chave);
                        $somenteXML = $this->sincronizacaoCTEFactory->somenteXML($chave);
                        
                        if($tipoNota) {
                    	    if(!isset($somenteXML[0]) && !isset($notaSincronizada[0])) {
                			    if(!isset($this->notasInexistentes[$chave])) {
                                    $this->notasInexistentes[$chave] = null;
                    	        }
                			}
                    	}
                        if(isset($notaSincronizada[0])) {
                        	$this->sincronizadas++;
                        }
                        else {
                        	if(isset($somenteXML[0])) {
                            	if(!isset($this->importacaoPendente[$chave])) {
                                    $this->importacaoPendente[$chave] = null;
                                }
                        	}
                        }
                    }
                }
            }
        }
        else if($cStat == 137) throw new \Exception($xMotivo, $cStat); //Nenhum documento localizado.
        else if($cStat == 656) throw new \Exception($xMotivo, $cStat); //Consumo indevido.
    }
    
    private function verificaXmlCte($cnpj, $tpamb, $nrnsu, $xml, $cdservico, $dsservico) {
        
        if(!$this->sincronizacaoCTEFactory->existeXmlNsuCTe($cnpj, $tpamb, $nrnsu)) {
            $this->sincronizacaoCTEFactory->insereXmlNsuCTe($cnpj, $tpamb, $nrnsu, $xml, $cdservico, $dsservico, $this->environment->getCdOperador(), $this->environment->getNrOrgTrab());
        }
    }
    
    private function atualizaNSUFaltante($cnpj, $tpAmb, $nsu) {
        $maxNrNsu = $this->sincronizacaoCTEFactory->maxNrNsu($cnpj, $tpAmb);
        
        if($maxNrNsu > 0) {
            $prxNrNSU = ltrim($nsu, "0");
            
            for($i = ($maxNrNsu + 1); $i <= ($prxNrNSU - 1); $i++) {
                
                $dadosCte = [
                    "NSU"          => str_pad($i, 15, "0", STR_PAD_LEFT), "DSSTATUS"     => "NSU inexistente no Sistema Teknisa",
                    "CDSTATUS"     => "999", "NRACESSOCTE"  => null,      "NRINSJURPESQ" => $cnpj, "IDTPAMBIENTE" => $tpAmb,
                    "DTEMISSAO"    => null,  "VRTOTALPREST" => null,      "VRRECEBER"    => null,  "NRINSJUREMIT" => null, 
                    "NMRAZSOCEMIT" => null,  "NRINSJURREME" => null,      "NMRAZSOCREME" => null,  "NRINSJUREXPE" => null, 
                    "NMRAZSOCEXPE" => null,  "NRINSJURRECE" => null,      "NMRAZSOCRECE" => null,  "NRINSJURDEST" => null, 
                    "NMRAZSOCDEST" => null,  "IDTOMADOR"    => null,      "NRINSJURTOMA" => null,  "NMRAZSOCTOMA" => null];
                
                $this->sincronizacaoCTEFactory->incluirNotaCte($dadosCte);
            }
        }
    }
    
    public function setAgendamentoManifestacao($nrorg, $useVpd, $newEntityManager, $newFileUploader) {
        if($useVpd) $this->entityManager = $newEntityManager;
        
        $this->environment->setNrOrg($nrorg);
        $this->environment->setNrOrgTrab($nrorg);
        $this->environment->setCurrentUserId('AUTOMATICO');
        $this->sincronizacaoCTEFactory->setAgendamentoManifestacao($nrorg, $useVpd, $newEntityManager, $newFileUploader);
    }
    
    private function replaceCharsForParserInverted($string) {
        $from = [
            "&lt;",
            "&gt;",
            "&amp;",
            "&quot;",
            "&#39;"
        ];
        
        $to = [
            "<",
            ">",
            "&",
            "\"",
            "''"
        ];
        return str_replace($from, $to, $string);
    }
    
    public function prestacaoServicoDesacordoCte($cte, $config) {
        if($cte['NRINSJURPESQ'] !== $cte['NRINSJURTOMA']) throw new \Exception('CTE_RECUSA_SERV_DESAC');
        
        $object_config = json_decode(json_encode($config));
        $cUF = substr($cte['NRACESSOCTE'], 0, 2);
        $uf = UFList::getUFByCode($cUF);
        
        try{
            if(!isset($this->cteTools)) {
        		$tools = $this->getToolsNFe($object_config->ie, $object_config->cnpj, json_encode($config));
        		$this->cteTools = $tools;
            }
            
            $cStatPermitidos = [134, 135, 136, 631];
            $this->aResposta = [];
            
            $resp = $this->cteTools->sefazManifesta($cte['NRACESSOCTE'], 610110, $cte['JUSTIFICATIVA'], 1, $uf);
            
            $this->aResposta = ReturnNFe::readReturnSefaz('RecepcaoEventoCTe', $resp);
            
            if(in_array($this->aResposta['cStat'], $cStatPermitidos)) {
                $nrevento = $this->sincronizacaoCTEFactory->maxEventoServDesac();
                $this->sincronizacaoCTEFactory->atualizaServDesac($cte['NRINSJURPESQ'], $cte['NRNSU'], $cte['IDTPAMBIENTE'], $cte['NRACESSOCTE']);
                $this->sincronizacaoCTEFactory->insereXmlServDesac($nrevento, $cte['NRACESSOCTE'], $resp, $this->environment->getNrOrgTrab());
                
                return ["SUCCESS" => $this->aResposta];
            }
            else return ["ERROR" => $this->aResposta['cStat'] . " - " . $this->aResposta['xMotivo']];
            
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage(), $e->getCode());
        }
    }
}
