<?php

namespace Teknisa\DF\Modules\Saida\Factories;

use Doctrine\ORM\EntityManager;

class Dfs61000TransmissaoNfseFactory {
    private $entityManager;
    private $connection;

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
        $this->connection = $this->entityManager->getConnection();
    }
    
    public function getCdReduCont($cdempresa, $cdcliente, $nrorg) {
        try {
            $sql = 
            "SELECT MAX(BF.CDREDUCONT) AS CDREDUCONT
              FROM BANCOPREF BF,
                CONTCORR CC
              WHERE (CC.CDEMPRESA  = :P_CDEMPRESA )
              AND (CC.CDREDUCONT   = BF.CDREDUCONT)
              AND (BF.CDCLIENTE    = :P_CDCLIENTE )
              AND (BF.NRORG        = :P_NRORG )
              AND (BF.IDPRIORBPREF = 'S')";

            $params = array(
                'P_CDEMPRESA'  => $cdempresa,
                'P_CDCLIENTE'  => $cdcliente,
                'P_NRORG'      => $nrorg
            );

            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getCdReduCont1($cdfilial, $nrlanctonf, $nrorg) {
        try {
            $sql = 
            "SELECT MAX(CDREDUGBFILI) AS CDREDUCONT
            FROM PARAMFILIAL
            WHERE (CDFILIAL =
              (SELECT CDFILIEMIT
              FROM NOTAFISCAL
              WHERE CDFILIAL = :P_CDFILIAL
              AND NRORG      = :P_NRORG
              AND NRLANCTONF = :P_NRLANCTONF))";

            $params = array(
                'P_CDFILIAL'   => $cdfilial,
                'P_NRORG'      => $nrorg,
                'P_NRLANCTONF' => $nrlanctonf
            );

            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getCdReduCont2($cdempresa, $nrorg) {
        try {
            $sql = 
            "SELECT MAX(P.CDREDUGBEMPR) AS CDREDUCONT
            FROM PARAMEMPRESA P
            WHERE (P.CDEMPRESA = :P_CDEMPRESA)
            AND P.NRORG        = :P_NRORG";

            $params = array(
                'P_CDEMPRESA'  => $cdempresa,
                'P_NRORG'      => $nrorg
            );

            $std = $this->connection->prepare($sql);
            $std->execute($params);
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getCarteira($cdempresa, $cdreducont, $nrorg) {
        try {
            $sql = <<<SQL
            SELECT MAX(E.CDCARTCOBR) AS CDCARTCOBR
            FROM EMPRBANC E LEFT JOIN CONTCORR C ON (E.CDBANCO = C.CDBANCO) AND (E.NRORG      = C.NRORG)
            WHERE (E.CDEMPRESA  = :P_CDEMPRESA) AND
            (C.CDREDUCONT = :P_CDREDUCONT) AND
	        (E.NRORG      = :P_NRORG)
SQL;

            $params = array(
                'P_CDEMPRESA'  => $cdempresa,
                'P_CDREDUCONT' => $cdreducont,
                'P_NRORG'      => $nrorg
            );

            $std = $this->connection->prepare($sql);
            $std->execute($params);
            
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getCedente($cdreducont, $nrorg) {
        try {
            $sql = <<<SQL
            SELECT * FROM CONTCORR WHERE CDREDUCONT = :P_CDREDUCONT AND NRORG = :P_NRORG
SQL;

            $params = array(
                'P_CDREDUCONT' => $cdreducont,
                'P_NRORG' => $nrorg
            );

            $std = $this->connection->prepare($sql);
            $std->execute($params);
            
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getXmlNotas($stringNota, $cdfilial, $nrorg) {
        try {
            $sql = 
            "SELECT AR.NRLANCTONF, AR.CDFILIAL, AR.DSARQXMLNFSE, NF.NRNOTAFISC
            FROM ARQXMLNFE AR, NOTAFISCAL NF
            WHERE AR.NRORG     = :P_NRORG 
            AND AR.CDFILIAL = :P_CDFILIAL
            AND AR.CDFILIAL = NF.CDFILIEMIT
            AND AR.NRLANCTONF = NF.NRLANCTONF
            AND AR.NRORG    = NF.NRORG
            AND AR.NRLANCTONF IN ";

            $sql.=$stringNota;
    
            $params = array(
                'P_NRORG' => $nrorg,
                'P_CDFILIAL' => $cdfilial
            );

            $std = $this->connection->prepare($sql);
            $std->execute($params);
            
            return $std->fetchAll();
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
     public function getLoteNotas($sqlConcat, $cdempresa) {
        $sql = "SELECT LOTE.CDEMPLOTERPS,
              LOTE.NRSEQLOTERPS,
              LOTE.NRINSJURIEST,
              EMP.NMRAZSOCEMPR,
              LOTE.CDINSCMUNI,
              LOTE.CDINSCESTA,
              LOTE.NRPROTOCOLORPS,
              LOTE.IDSTATUSLOTERPS,
              LOTE.DTENVIOLOTERPS,
              LOTE.NRORG,
              RPS.CDSERIERPS,
              COUNT(RPS.NRRPS) AS QTDERPS
            FROM LOTERPS LOTE
            INNER JOIN NUMERORPS RPS
            ON LOTE.CDEMPLOTERPS  = RPS.CDEMPLOTERPS
            AND LOTE.NRINSJURIEST = RPS.NRINSJURIEST
            AND LOTE.NRSEQLOTERPS = RPS.NRSEQLOTERPS
            AND LOTE.NRORG        = RPS.NRORG
            INNER JOIN EMPRESA EMP
            ON LOTE.CDEMPLOTERPS = EMP.CDEMPRESA
            AND LOTE.NRORG       = EMP.NRORG
            AND LOTE.NRSEQLOTERPS IN
          (SELECT DISTINCT NRSEQLOTERPS
          FROM NOTAFISCAL
          WHERE NRLANCTONF IN";
          $sql.=$sqlConcat;
          $sql.=
            ") AND LOTE.CDEMPLOTERPS = :P_CDEMPRESA
            GROUP BY LOTE.CDEMPLOTERPS,
              LOTE.NRSEQLOTERPS,
              LOTE.NRINSJURIEST,
              EMP.NMRAZSOCEMPR,
              LOTE.CDINSCMUNI,
              LOTE.CDINSCESTA,
              LOTE.NRPROTOCOLORPS,
              LOTE.IDSTATUSLOTERPS,
              LOTE.DTENVIOLOTERPS,
              LOTE.NRORG,
              RPS.CDSERIERPS
            ORDER BY IDSTATUSLOTERPS ASC,
              DTENVIOLOTERPS DESC";
            
            $params = array(
                'P_CDEMPRESA' => $cdempresa
            );

            $std = $this->connection->prepare($sql);
            $std->execute($params);
            
            return $std->fetchAll();

    }
    
    public function updateDataEmissao($cdfilial , $nrlanctonf, $nrorg,$dtatual ) {
        try {
            $sql = <<<SQL
            UPDATE NOTAFISCAL 
            SET  DTEMISSAO = :P_DTATUAL,
                 NRNOTAFISC = '999999999'
            WHERE CDFILIAL = :P_CDFILIAL 
                  AND NRLANCTONF =  :P_NRLANCTONF 
                  AND  NRORG = :P_NRORG
SQL;

            $params = array(
                'P_CDFILIAL'  => $cdfilial,
                'P_NRLANCTONF' => $nrlanctonf,
                'P_NRORG'      => $nrorg,
                'P_DTATUAL'   => $dtatual
            );

            $std = $this->connection->prepare($sql);
            
            
            
            return $std->execute($params);
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    public function updateStatus($cdfilial , $nrlanctonf, $nrorg) {
        try {
            $sql = <<<SQL
            UPDATE NOTAFISCAL 
            SET  IDBLOQUEADO = 'S',
                 IDSTATUS = 'A'
            WHERE CDFILIAL = :P_CDFILIAL 
                  AND NRLANCTONF =  :P_NRLANCTONF 
                  AND  NRORG = :P_NRORG
SQL;

            $params = array(
                'P_CDFILIAL'  => $cdfilial,
                'P_NRLANCTONF' => $nrlanctonf,
                'P_NRORG'      => $nrorg
            );

            $std = $this->connection->prepare($sql);
            
            
            
            return $std->execute($params);
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getContareceber ($cdfilial,$nrlanctonf, $nrorg ){
         try {
            $sql = <<<SQL
           SELECT CDFILIBOLREC, NRLANBOLREC, CONTARECEBER.*
              FROM CONTARECEBER
             WHERE CDFILIAL = :P_CDFILIAL
               AND NRLANCTONF = :P_NRLANCTONF
               AND NRORG = :P_NRORG
SQL;

            $params = array(
                'P_CDFILIAL'  => $cdfilial,
                'P_NRLANCTONF' => $nrlanctonf,
                'P_NRORG'      => $nrorg
            );

            $std = $this->connection->prepare($sql);
            
            $std->execute($params);
            return $std->fetchAll(); 
          
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosSmtpNfse($nrorg){
         try {
            $sql = "SELECT NVL (B.DSLOGINSMTPGENTR,E.DSLOGINSMTP) AS DSLOGINSMTP, NVL(B.DSSMTPEMAILFGENTR, E.DSSMTPEMAILF)AS DSSMTPEMAILF,
				NVL(B.CDSENHEMAILFGENTR,E.CDSENHEMAILF) AS CDSENHEMAILF , NVL (B.IDUTTLSEMAILGENTR, E.IDUTTLSEMAIL) AS IDUTTLSEMAIL,
				NVL (B.IDUTSSLEMAILGENTR,E.IDUTSSLEMAIL)AS IDUTSSLEMAIL, P.DSEMAILREMETXML AS REMETENTE
                  FROM PARAMINTEGRA P, PARGERALDFE B, ENDEFILI E
                  JOIN FILIAL F ON (E.CDFILIAL = F.CDFILIAL)
                               AND (E.NRORG = F.NRORG)
                 WHERE IDTPENDEFILI = 'P'
                   AND F.CDFILIAL = (SELECT MAX(CDFILIAL)
                                       FROM FILIAL
                                      WHERE NRORG = :P_NRORG)
                  AND P.NRORG = E.NRORG
                  AND B.NRORG = E.NRORG";

            $params = array(
                'P_NRORG'      => $nrorg
            );

            $std = $this->connection->prepare($sql);
            
            $std->execute($params);
            return $std->fetchAll(); 
          
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
}